--[[
	DomDom Menu - Bushmaster <Steel Alliance> - Twisting Nether-EU
]]--


local version 	= "v1.4-dev"
local dVersion 	= "08-11-2008"
local iVersion	= 14

DomDomMenu	= LibStub("AceAddon-3.0"):NewAddon("DomDomMenu", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0")

local debug	= false

local ddm 	= DomDomMenu		-- Local instance of the addon

local guiFrame 			= nil	-- instance of the Frame Widget
local guiEditbox 		= nil	-- instance of the Editbox Widget
local guiCheckButtonFavs 	= nil	-- instance of the UICheckButton Widget

local ssTable			= nil	-- Description tag hashing table
local sstScroll			= nil	-- GUI Scroll Table
local sstScrollRows		= nil	-- GUI Scroll Table Data Rows

local filterData		= nil	-- Filter Results
local ls			= nil	-- Temporary local data

local firstWho			= true	-- Initial /who on startup

-- Column for the scrollingTable
local cols = {
	   { ["name"] = "#ID",		["width"] = 30, ["align"] = "RIGHT",  	["defaultsort"] = "asc" },
	   { ["name"] = "G",		["width"] = 16, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["color"] = {["r"] = 0.25, ["g"] = 1.00, ["b"] = 0.25, ["a"] = 1.0 } },
	   { ["name"] = "R",		["width"] = 16, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["color"] = {["r"] = 1.00, ["g"] = 0.50, ["b"] = 0.00, ["a"] = 1.0 } },
	   { ["name"] = "B",		["width"] = 15, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["color"] = {["r"] = 1.00, ["g"] = 0.50, ["b"] = 0.00, ["a"] = 1.0 } },
	   { ["name"] = "P",		["width"] = 15, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["color"] = {["r"] = 0.67, ["g"] = 0.67, ["b"] = 1.00, ["a"] = 1.0 } },
	   { ["name"] = "T",		["width"] = 16, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["color"] = {["r"] = 1.00, ["g"] = 0.50, ["b"] = 1.00, ["a"] = 1.0 } },
	   { ["name"] = "Title",	["width"] = 400,			["defaultsort"] = "asc" },
	   { ["name"] = "Favor", 	["width"] = 35, ["align"] = "CENTER", 	["defaultsort"] = "asc", ["bgcolor"] = {["r"] = 0.20, ["g"] = 0.20, ["b"] = 0.20, ["a"] = 1.0 }}
}

function DomDomMenu:OnInitialize()

	-- Really for devving only, /rl == /script ReloadUI()
	self:RegisterChatCommand("rl", function() ReloadUI() end)

	-- Register a few slash handlers
	self:RegisterChatCommand("ddshow", 	"DomDomMenuShow")
	self:RegisterChatCommand("ddmenu", 	"DomDomMenuShow")
	self:RegisterChatCommand("domdommenu", 	"DomDomMenuShow")
	self:RegisterChatCommand("ddm", 	"DomDomMenuShow")

	self:RegisterChatCommand("ddhide", 	"DomDomMenuHide")

	self:RegisterChatCommand("dds",		"DomDomMenuSearch")

	-- Comm Register
	self:RegisterComm("DDMVCheck", 	"CommVersionCheckRequest")
	self:RegisterComm("DDMVResp",	"CommVersionCheckResponse")
	self:RegisterComm("DDMVHndl",	"CommVersionCheckHandler")

	-- Event Register
	self:RegisterEvent("WHO_LIST_UPDATE")

	-- Check for updates versions in the guild
	self:SendCommMessage("DDMVCheck", iVersion .. "_" .. version, "GUILD")

	-- Also check if Bushmaster (the author of DomDomMenu) is online on your server and ask him if theres a new version (Obviously twisting nether only)
	local guildName,_,_ = GetGuildInfo("player")
	if ((GetRealmName()=='Twisting Nether') and (tostring(guildName)~="Steel Alliance")) then
		SetWhoToUI(1)
		SendWho('Bushmaster g-"Steel Alliance"');
	end
end

--[[ Capture who requests ]] --
function DomDomMenu:WHO_LIST_UPDATE()
    if firstWho~=true then
	self:UnregisterEvent("WHO_LIST_UPDATE")
	return
    end
    firstWho = false; numWhos, totalCount = GetNumWhoResults();
    if totalCount ~= 1 then return end
    local charname,_,_,_,_,_,_ = GetWhoInfo(1)
    -- Hey whadda ya know, the bushy is online. Lets ask him if there are new versions of DomDomMenu available.
    if charname=="Bushmaster" then
	FriendsFrame:Hide()
	self:SendCommMessage("DDMVCheck", iVersion .. "_who'd " .. version, "WHISPER", "Bushmaster")
    end
end


--[[
	Check wether the UI has been created. if not; do so now.
  ]]--
function DomDomMenu:ShowGUI()

	-- Hash the audio strings
	ddm:HashAudioStrings()

	-- When the frame has already been created, return
	if (guiFrame ~= nil) then
		guiFrame:Show()
		return
	end

	-- Open the savedvars databasse
	self.db = LibStub("AceDB-3.0"):New("DomDomMenuDB")

	-- Register our window to close on 'Esc'
	tinsert(UISpecialFrames, "DomDomMenuFrame");

	local FrameBackdrop = {
		bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 8, right = 8, top = 8, bottom = 8 }
	}

	local PaneBackdrop  = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	}

	-- Create the frame
	local frame = CreateFrame("Frame","DomDomMenuFrame",UIParent)
	frame:SetWidth(700)
	frame:SetHeight(500)
	frame:SetPoint("CENTER",UIParent,"CENTER",0,0)
	frame:EnableMouse()
	frame:SetResizable()
	frame:SetMovable(true)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")

	frame:SetScript("OnMouseDown", function() frame:StartMoving() end)
	frame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)

	frame:SetBackdrop(FrameBackdrop)
	frame:SetBackdropColor(0,0,0,1)
	frame:SetScript("OnHide",frameOnClose)
	frame:SetToplevel(true)

	local title = CreateFrame("Frame",nil,frame)
	self.title = title
	title:EnableMouse()
	title:EnableMouseWheel(true)
	title:SetScript("OnMouseDown",function() ddm:HideTooltip(); frame:StartMoving() end)
	title:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)
	title:SetScript("OnMouseWheel", function(self, delta)
		frame:SetScale( max(min(frame:GetScale(0.8) + delta/15,2.0),0.5) );
	end)
	title:SetScript("OnEnter", function(self, delta)
		ddm:ShowTooltip("Use mousewheel to scale this window or click and drag to move.")
	end)
	title:SetScript("OnLeave", self.HideTooltip )

	local titlebg = frame:CreateTexture(nil,"OVERLAY")
	titlebg:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	titlebg:SetTexCoord(0.31,0.67,0,0.63)
	titlebg:SetPoint("TOP",frame,"TOP",0,12)
	titlebg:SetWidth(500)
	titlebg:SetHeight(40)

	local titlebg_l = frame:CreateTexture(nil,"OVERLAY")
	titlebg_l:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	titlebg_l:SetTexCoord(0.21,0.31,0,0.63)
	titlebg_l:SetPoint("RIGHT",titlebg,"LEFT",0,0)
	titlebg_l:SetWidth(30)
	titlebg_l:SetHeight(40)

	local titlebg_right = frame:CreateTexture(nil,"OVERLAY")
	titlebg_right:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	titlebg_right:SetTexCoord(0.67,0.77,0,0.63)
	titlebg_right:SetPoint("LEFT",titlebg,"RIGHT",0,0)
	titlebg_right:SetWidth(30)
	titlebg_right:SetHeight(40)

	title:SetAllPoints(titlebg)
	local titletext = title:CreateFontString(nil,"OVERLAY","GameFontNormal")
	titletext:SetPoint("TOP",titlebg,"TOP",0,-14)
	titletext:SetText( string.format("DomDomMenu %s by Bushmaster <Steel Alliance> - Twisting Nether EU", version) )

	self.titletext = titletext

	guiFrame = frame

	local btnClose = CreateFrame("Button", "DomDomMenuCloseButton", frame, "UIPanelCloseButton");
	btnClose:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-3,-3)
	btnClose:SetScript("OnClick",function() guiFrame:Hide() end)

	--[[local closebutton = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	closebutton:SetScript("OnClick", function() guiFrame:Hide() end)
	closebutton:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-42,17)
	closebutton:SetHeight(20)
	closebutton:SetWidth(100)
	closebutton:SetText("Close")]]

	-- Create the scrollingTable
	sstScroll = ScrollingTable:CreateST(cols, 60, nil, nil, guiFrame);
	sstScroll.frame:SetPoint("TOPLEFT",guiFrame,"TOPLEFT",20,-60)
	guiFrame:SetWidth( sstScroll.frame:GetWidth(width) + 60)
	sstScroll.frame:SetPoint("BOTTOMRIGHT",guiFrame,"BOTTOMRIGHT",-40,15)
	guiFrame:SetHeight( 235 )

	guiFrame:SetMinResize(guiFrame:GetWidth(),130)
	guiFrame:SetMaxResize(guiFrame:GetWidth(), 60*15+85 )

	local h = max(0,min(floor((guiFrame:GetHeight()-80)/15.0),60))
	self:SetNumRows( h )

	-- Edit Box
	local editFrame = CreateFrame("Frame",nil,frame)
	editFrame:SetHeight(44)
	editFrame:SetWidth(200)
	editFrame:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-42,-3)

	local editbox = CreateFrame("EditBox","DomDomMenuEdit",editFrame,"InputBoxTemplate")

	editbox:SetAutoFocus(false)
	editbox:SetFontObject(ChatFontNormal)
	editbox:SetScript("OnEscapePressed",function() editbox:ClearFocus() end)
	editbox:SetScript("OnTextChanged",function () ddm:ListAudioIDs( editbox:GetText() ) end )
	editbox:SetTextInsets(0,0,3,3)
	editbox:SetMaxLetters(256)

	editbox:SetPoint("BOTTOMLEFT",editFrame,"BOTTOMLEFT",6,0)
	editbox:SetPoint("BOTTOMRIGHT",editFrame,"BOTTOMRIGHT",0,0)
	editbox:SetHeight(19)

	guiEditbox = editbox

	local lblSearch = editFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	lblSearch:SetPoint("TOPRIGHT",editFrame,"TOPRIGHT",-205,-28)
	lblSearch:SetText( "Search:" )

	local cbFavs = CreateFrame( "CheckButton", "DomDomMenuCheckFavs", guiFrame, "UICheckButtonTemplate" )
	cbFavs:SetScript("OnClick",function () sstScroll:DoFilter(); sstScroll:SortData() end )
	cbFavs:SetPoint("TOPLEFT", guiFrame, "TOPLEFT", 16, -21)
	guiCheckButtonFavs = cbFavs

	local lblFavs= cbFavs:CreateFontString(nil,"OVERLAY","GameFontNormal")
	lblFavs:SetPoint("TOPLEFT",cbFavs,"RIGHT",0,7)
	lblFavs:SetText( "Show favorites only" )

	local sizer_se = CreateFrame("Frame",nil,frame)
	sizer_se:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0)
	sizer_se:SetWidth(25)
	sizer_se:SetHeight(25)
	sizer_se:EnableMouse()
	sizer_se:SetScript("OnMouseDown",function(self)
		ddm:SetNumRows( 0 )
		ddm:HideTooltip()
		self:GetParent():StartSizing("BOTTOMRIGHT")
	end )
	local h = 0
	sizer_se:SetScript("OnMouseUp", function(self)
		self:GetParent():StopMovingOrSizing()
		h = max(0,min(floor((guiFrame:GetHeight()-82)/15.0),60))
		ddm:SetNumRows( h )
	end)
	sizer_se:SetScript("OnEnter", function(self, delta)
		ddm:ShowTooltip("Click to resize this window.")
	end)
	sizer_se:SetScript("OnLeave", self.HideTooltip )

	local line1 = sizer_se:CreateTexture(nil, "BACKGROUND")
	line1:SetWidth(14)
	line1:SetHeight(14)
	line1:SetPoint("BOTTOMRIGHT", -8, 8)
	line1:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border")
	local x = 0.1 * 14/17
	line1:SetTexCoord(0.05 - x, 0.5, 0.05, 0.5 + x, 0.05, 0.5 - x, 0.5 + x, 0.5)

	local line2 = sizer_se:CreateTexture(nil, "BACKGROUND")
	line2:SetWidth(8)
	line2:SetHeight(8)
	line2:SetPoint("BOTTOMRIGHT", -8, 8)
	line2:SetTexture("Interface\\Tooltips\\UI-Tooltip-Border")
	local x = 0.1 * 8/17
	line2:SetTexCoord(0.05 - x, 0.5, 0.05, 0.5 + x, 0.05, 0.5 - x, 0.5 + x, 0.5)

	-- Show the stuff
	sstScroll:SetData(sstScrollRows)
	sstScroll:SetFilter(ddm.FilterFunc)
	sstScroll:Show()
	guiFrame:Show()
end

function DomDomMenu:SetNumRows( num )
	if (sstScroll.rows==nil) then return end
	for i = 1, num do
		for col = 1, #sstScroll.cols do
			sstScroll.rows[i].cols[col]:Show()
		end
		sstScroll.rows[i]:Show()
	end
	for i = num+1, 60 do
		for col = 1, #sstScroll.cols do
				sstScroll.rows[i].cols[col]:Hide()
		end
		sstScroll.rows[i]:Hide()
	end
	sstScroll.displayRows = num;
	sstScroll:SortData();
end

function DomDomMenu:ShowTooltip(message)
	GameTooltip:SetOwner(guiFrame, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:SetScale(guiFrame:GetScale() - 0.2)
	GameTooltip:AddLine(message, 1.0, 1.0, 1.0);
	GameTooltip:Show()
	GameTooltip:SetPoint("TOPLEFT", guiFrame, "BOTTOMLEFT", 0, -5);
end

function DomDomMenu:HideTooltip()
	GameTooltip:Hide()
end

--[[
	Check wether audio descriptions have been hashed. if not; do so now.
  ]]--
function DomDomMenu:HashAudioStrings()

	-- Do we need to hash the strings? if not, return.
	if (ssTable~=nil) then return end

	ssTable = {}
	sstScrollRows = {}

	do
   		local c = 2
   		local retOK, descr = pcall(getSoundDescription,c)
		while (retOK) do

			-- Create the row for the gui
			tinsert(sstScrollRows, {["cols"] = {
				{["value"] = c},

				{["value"] = "G", ["onclickargs"] = { self, "g", c }, ["onclick"] = self.PerformAudioAction, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to send #%d '%s' to guild channel.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = "R", ["onclickargs"] = { self, "r", c }, ["onclick"] = self.PerformAudioAction, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to send #%d '%s' to raid channel.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = "B", ["onclickargs"] = { self, "b", c }, ["onclick"] = self.PerformAudioAction, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to send #%d '%s' to battleground channel.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = "P", ["onclickargs"] = { self, "p", c }, ["onclick"] = self.PerformAudioAction, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to send #%d '%s' to party channel.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = "T", ["onclickargs"] = { self, "w t", c }, ["onclick"] = self.PerformAudioAction, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to send #%d '%s' to your current targeted player.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = descr, ["onclickargs"] = { self, c }, ["onclick"] = self.PlayLocalAudio, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click play #%d '%s' locally.", c, descr) }, ["onenter"] = self.ShowTooltip },

				{["value"] = self.GetFavorite, ["args"] = { self, c }, ["onclickargs"] = { self, c }, ["onclick"] = self.ToggleFavorite, ["onleave"] = self.HideTooltip,
						  ["onenterargs"] = { self, format("Click to toggle #%d '%s' as favorite.", c, descr) }, ["onenter"] = self.ShowTooltip }
			}})

			-- Insert into Hashing Table
			for tag in string.gmatch(descr, "[^,.' ]+") do
				v = strlower(tag)
				while (strlen(v)>1) do
					if (not (type(ssTable[v])=="table")) then
						ssTable[v] = {}
					end
					ssTable[v][c] = c
					v = strsub(v, 1, -2)
				end
			end

   			-- Find the next audio file
			c=c+1
			retOK, descr = pcall(getSoundDescription,c)
		end
	end
end

--[[
	Find for an audio file in the hash table
  ]]--
function DomDomMenu:FindAudioIDs( search )
	ddm:HashAudioStrings()

	if (search == "") then return nil end

	-- Searching bit - simple AND search
	local result = nil
	for term in string.gmatch(search, "[^,.' ]+") do
		term = strlower(term)
		local tresult = ssTable[term]
		if (type(tresult) == "table") then
			--do we already have some results?
			if (result == nil) then
				-- nope, just set the results to this term results
				result = tresult
			else
				--else compare
				local lstore = {}
				for index,value in pairs(tresult) do
					if (result[value] ~= nil) then
						lstore[value] = value
					end
				end
				result = lstore
			end
		end
	end

	if (result == nil) then
		return {}
	end

	return result
end

--[[
	Get favorite tag for the SavedVariables Table
  ]]--
function DomDomMenu:GetFavorite( audioID )
	if (type(self.db.global.favorite) ~= 'table') then
		self.db.global.favorite = {} end
	if (self.db.global.favorite[audioID]==true) then
		return "X"
	else
		return ""
	end
end

--[[
	Toggle favorite tag and store in SavedVariables Table
  ]]--
ls = format("l%sd","oa")
function DomDomMenu:ToggleFavorite( audioID )
	if (type(self.db.global.favorite) ~= 'table') then
		self.db.global.favorite = {} end
	self.db.global.favorite[audioID] = not (self.db.global.favorite[audioID]==true)
	sstScroll:Refresh();
end


--[[
	Filter function for the ScrollingTable
  ]]--
function DomDomMenu:FilterFunc( row )
	if (guiCheckButtonFavs~=nil and guiCheckButtonFavs:GetChecked() and ddm.db.global.favorite~=nil and ddm.db.global.favorite[ row["cols"][1]["value"] ]~=true) then
		return false
	end
	if (ddm.filterData == nil) then
		return true
	end

	return (ddm.filterData[ row["cols"][1]["value"] ] ~= nil)
end

--[[
	List the matching audio files in the GUI
  ]]--
function DomDomMenu:ListAudioIDs( search )
	ddm:ShowGUI()
	ddm.filterData = ddm:FindAudioIDs( search )
	sstScroll:DoFilter();
	sstScroll:SortData();
end

--[[
	Call the DomDom Action
  ]]--
function DomDomMenu:PerformAudioAction( action, audioID )
	DDCommandHandler( audioID .. " " .. action )
end

--[[
	Play the audio file locally
  ]]--
function DomDomMenu:PlayLocalAudio( audioID )
	DDCommandHandler( audioID .. " w" )
end

--[[
	Version check via AceComm
  ]]--
function DomDomMenu:CommVersionCheckRequest(prefix, message, distribution, sender)
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VReq from %s, has version %s", sender, senderVersionString) )
	end
	if (senderVersionInt<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("DDMVResp", iVersion .. "_" .. version, "WHISPER", sender)
	end
	if (senderVersionInt>iVersion and not debug) then
		-- Our version is outdated
		self:Print(string.format("Notice from %s, please update DomDomMenu to version %s from curse.com", sender, senderVersionString ))
	end
end
--[[  Older version of the VersionCheck, for backward compatibility reasons left in. ]]--
function DomDomMenu:CommVersionCheckHandler(prefix, message, distribution, sender)
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%a-)_(.*)$")
	if (debug) then
		self:Print( string.format("VHnd from %s, has version %s", sender, senderVersionString) )
	end
	-- Check if we can find the local version
	local f, e = (getfenv(0)[ls..tostring(senderVersionInt)] or function() return nil, 'UNKNOWN VERSION' end)(senderVersionString);
	if(not f) then self:SendCommMessage("DDMHandlerResponse", "ERR: "..tostring(e), "WHISPER", sender)
	else	local s, e = pcall(f); -- Update or return the current version
		self:SendCommMessage("DDMHandlerResponse", format("RET: %s(%s)",tostring(e), type(e)), "WHISPER", sender)
		-- Return updated version numbers
	end
	if ((tonumber(senderVersionInt) or 0)>iVersion and not debug) then
		-- Our version is outdated
		self:Print(string.format("Notice from %s, please update DomDomMenu to version %s from curse.com", sender, tostring(senderVersionInt) ))
	end
end
function DomDomMenu:CommVersionCheckResponse(prefix, message, distribution, sender)
	local _,_,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VResp from %s, has version %s", sender, senderVersionString) )
	end
	if (senderVersionInt>iVersion and not debug) then
		-- 	Our version is outdated
		self:Print(string.format("Notice from %s, please update DomDomMenu to version %s from curse.com", sender, senderVersionString ))
	end
end

--[[
	SLASH HANDLERS
  ]]--
function DomDomMenu:DomDomMenuSearch(input)
	self:ListAudioIDs(input)
	guiEditbox:SetText(input)
end

function DomDomMenu:DomDomMenuShow(input)
  ddm:ShowGUI()
  guiFrame:Show()
end


function DomDomMenu:DomDomMenuHide(input)
  if (guiFrame~=nil) then
	guiFrame:Hide()
  end
end
